package mngr

import (
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"github.com/pkg/errors"
	"gitlab.smartblocklab.com/smartcontracts/common/model/cryptocurrencyCC"
)

func (this *CryptocurrencyManagerImpl) AddCryptocurrencies(daemonName string, cryptocurrencyListInput cryptocurrencyCC.CryptocurrencyListInput) (interface{}, error) {
	chaincode.GlobalLogger.Infof("AddCryptocurrencies [daemonName: %s]", daemonName)

	// проверка последнего времени страбатывания таймера
	invokeTime := this.GetTime()

	// TODO раскоментировать перед релизом
	//err := this.checkLastImportTime(invokeTime)
	//if err != nil {
	//	return nil, err
	//}

	// получить последние импортированные ключи
	lastImportKeys, err := this.getLastImportKeys()
	if err != nil {
		return nil, errors.Errorf("Failed to GetLastImportKeys, failed due to %s", err)
	}

	var transactionResultArray []cryptocurrencyCC.CryptocurrencyResult

	chaincode.GlobalLogger.Infof("cryptocurrencyListInput.Cryptocurrencies len: ", len(cryptocurrencyListInput.Cryptocurrencies))
	for _, cryptocurrency := range cryptocurrencyListInput.Cryptocurrencies {
		// добавляем криптовалюты для выгрузки средств из внешних систем
		// составляем список lastImportKeys из транзакций по криптовалютам, которые успешно дабавлены в систему
		transactionResult := this.AddCryptocurrency(daemonName, *lastImportKeys, cryptocurrency)

		if transactionResult.Status == cryptocurrencyCC.Ok {
			// запись информации о транзакции
			lastImportKeys.MapDaemonNameKeys = this.AddToMapDaemonNameKeys(lastImportKeys.MapDaemonNameKeys, daemonName, cryptocurrency.ExternalTransactionID)
		}
		// собираем список результатов по всем криптовалютам
		transactionResultArray = append(transactionResultArray, transactionResult)
	}

	// запись последних импортированных ключей
	err = this.saveLastImportKeys(*lastImportKeys)
	if err != nil {
		return nil, err
	}

	// сохранение времени срабатывания шедулера
	err = this.saveLastImportTime(invokeTime)
	if err != nil {
		return nil, err
	}

	importTransactionResult := cryptocurrencyCC.ImportCryptocurrencyResult{
		CryptocurrencyResultList: transactionResultArray,
	}

	return importTransactionResult, nil
}
