package mngr

import (
	"fmt"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"gitlab.smartblocklab.com/smartcontracts/common/model/cryptocurrencyCC"
	"gitlab.smartblocklab.com/smartcontracts/common/model/requests"
	"gitlab.smartblocklab.com/smartcontracts/common/model/accountCC"
)

// добавляет одной внешнюю транзакцию
func (this *CryptocurrencyManagerImpl) AddCryptocurrency(daemonName string, lastImportKeys cryptocurrencyCC.LastImportKeys, cryptocurrencyInput cryptocurrencyCC.CryptocurrencyInput) cryptocurrencyCC.CryptocurrencyResult {
	chaincode.GlobalLogger.Infof("AddCryptocurrency [daemonName: %s, cryptocurrencyInput: %s]", daemonName, cryptocurrencyInput.ToString)
	transactionResult := cryptocurrencyCC.CryptocurrencyResult{
		ExternalTransactionID: cryptocurrencyInput.ExternalTransactionID,
		Status:                cryptocurrencyCC.Ok,
		CommonMessage:         "",
		DetailMessage:         "",
	}

	if len(cryptocurrencyInput.ExternalTransactionID) == 0 {
		transactionResult.Status = cryptocurrencyCC.Error
		transactionResult.CommonMessage = "ExternalTransactionID can't be null"

		return transactionResult
	}

	cryptocurrencyStore, err := this.getCryptocurrencyStore(cryptocurrencyInput.ExternalTransactionID)
	if err != nil {
		transactionResult.Status = cryptocurrencyCC.Error
		transactionResult.CommonMessage = fmt.Sprintf("Failed to get getCryptocurrencyStore by transactionId %s", cryptocurrencyInput.ExternalTransactionID)
		transactionResult.DetailMessage = err.Error()

		return transactionResult

	}
	if cryptocurrencyStore != nil {
		transactionResult.Status = cryptocurrencyCC.Error
		transactionResult.CommonMessage = fmt.Sprintf("Cryptocurrency with transactionId %s already exists", cryptocurrencyInput.ExternalTransactionID)

		return transactionResult
	}

	// если транзакция уже не была загружена шедулером ранее и находится в обработке
	if !this.isKeyExists(daemonName, lastImportKeys, cryptocurrencyInput) {

		inAccountInfo, err := this.GetAccountInfoByAccountID(&requests.Id{Id: cryptocurrencyInput.InExternalAccountID})
		if err != nil {
			transactionResult.Status = cryptocurrencyCC.Error
			transactionResult.CommonMessage = "Check accountLoader"
			transactionResult.DetailMessage = err.Error()

			return transactionResult
		}

		// TODO не реализовано в BTC
		//outAccountInfo, err := t.accountLoader(cryptocurrencyInput.OutExternalAccountID, stub)
		//if err != nil {
		//	transactionResult.Status = model.Error
		//	transactionResult.CommonMessage = "Check accountLoader"
		//	transactionResult.DetailMessage = err.Error()
		//	return transactionResult
		//}

		// TODO получаем из канала externalwalletchannel по организации список ip адресов для разных криптовалют
		// impl
		// impl
		// impl

		// TODO проверка транзакции на сервере криптовалюты
		err = this.checkCryptocurrency(cryptocurrencyInput)
		if err != nil {
			transactionResult.Status = cryptocurrencyCC.Error
			transactionResult.CommonMessage = "Check cryptocurrency"
			transactionResult.DetailMessage = err.Error()

			// добавлем информацию по ошибке проверки транзакции во внешней системе

			return transactionResult
		}

		cryptocurrencyStore := &cryptocurrencyCC.CryptocurrencyStore{
			Time:                  cryptocurrencyInput.Time,
			ExternalTransactionID: cryptocurrencyInput.ExternalTransactionID,
			OutAccountInfo:        nil,
			InAccountInfo: &accountCC.UserAccountInfo{
				UserID: inAccountInfo.UserID,
				TxnAccountInfo: &accountCC.TxnAccountInfo{
					AccountID: inAccountInfo.AccountID,
					WalletID:  inAccountInfo.WalletID,
				},
			},
			Amount: cryptocurrencyInput.Amount,
		}

		_, err = this.Save(cryptocurrencyStore)
		if err != nil {
			transactionResult.Status = cryptocurrencyCC.Error
			transactionResult.CommonMessage = "Save cryptocurrency store"
			transactionResult.DetailMessage = err.Error()

			// добавлем информацию по ошибке сохранения в блокчейн
			return transactionResult
		}
	} else {
		errMsg := fmt.Sprintf("TransactionID %s already exists in map 'lastImportKeys'", cryptocurrencyInput.ExternalTransactionID)

		transactionResult.Status = cryptocurrencyCC.Error
		transactionResult.CommonMessage = "Is key exists"
		transactionResult.DetailMessage = errMsg
	}

	return transactionResult
}

func contains(s []string, e string) bool {
	for _, a := range s {
		if a == e {
			return true
		}
	}
	return false
}
