package mngr

import (
	"encoding/binary"
	"fmt"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"github.com/pkg/errors"
	"gitlab.smartblocklab.com/smartcontracts/common/model/cryptocurrencyCC"
)

// проверка времени отработки шедулера
func (this *CryptocurrencyManagerImpl) checkLastImportTime(currentInvokeTime int64) error {
	lastImportTimeByteArray, err := this.GetState(cryptocurrencyCC.LAST_IMPORT_TIME)
	if err != nil {
		return err
	}

	var lastImportTime int64

	if lastImportTimeByteArray == nil {
		lastImportTime = currentInvokeTime - cryptocurrencyCC.WAITING_SECONDS
	} else {
		lastImportTime = int64(binary.LittleEndian.Uint64(lastImportTimeByteArray))
	}

	if currentInvokeTime < lastImportTime+cryptocurrencyCC.WAITING_SECONDS {
		// считаем сколько нам осталось еще подождать
		needToWait := cryptocurrencyCC.WAITING_SECONDS - (currentInvokeTime - lastImportTime)
		needToWaitMsg := fmt.Sprintf("needToWait : %s", needToWait)
		chaincode.GlobalLogger.Info(needToWaitMsg)
		return errors.Errorf("You need to wait to %d seconds", needToWait)
	}

	return nil
}
