package mngr

import (
	"fmt"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"github.com/pkg/errors"
	"gitlab.smartblocklab.com/smartcontracts/common/model/transactionCC"
	"gitlab.smartblocklab.com/smartcontracts/common/model/requests"
)

// метод синхронизирует последние импортированные ключей ключей по демону
//
// 1. достает ExternalTransactionID добавленных транзакций в cryptocurrencychannel
// 2. проверяет есть ли транзакция для такого ExternalTransactionID (поиск по operationId)
// 3. если находятся транзакции тогда мы их удаляем из списка импортированных ключей по демону
func (this *CryptocurrencyManagerImpl) ClearTransferredCryptocurrencyList(daemonName *requests.Id) (*transactionCC.CheckExistsResult, error) {
	if len(daemonName.Id) == 0 {
		return nil, errors.New("daemonName is empty")
	}

	// получить последние импортированные ключи всех демонов по расписанию
	lastImportKeys, err := this.getLastImportKeys()
	if err != nil {
		return nil, errors.Errorf("Failed to GetLastImportKeys, failed due to %s", err)
	}

	checkExistsResult := &transactionCC.CheckExistsResult{}

	// найти среди списка текущий демон
	daemonKeys, ok := lastImportKeys.MapDaemonNameKeys[daemonName.Id]
	if len(daemonKeys) == 0 || !ok {
		return checkExistsResult, nil
	}

	// составить список из ExternalTransactionID которые были добавлены в канал cryptocurrencychannel
	syncRequest := transactionCC.SyncRequest{}

	chaincode.GlobalLogger.Infof("daemonKeys")
	for externalTransactionID := range daemonKeys {
		chaincode.GlobalLogger.Infof("ExternalTransactionID: %s", externalTransactionID)

		cryptocurrencyStore, err := this.getCryptocurrencyStore(externalTransactionID)
		if err == nil && cryptocurrencyStore != nil {
			chaincode.GlobalLogger.Infof("Success getCryptocurrencyStore by ExternalTransactionID: %s", externalTransactionID)

			externalTransactionInfo := transactionCC.ExternalTransactionInfo{}
			operationID := fmt.Sprintf("%s.%s", cryptocurrencyStore.InAccountInfo.AccountID, cryptocurrencyStore.ExternalTransactionID)
			chaincode.GlobalLogger.Infof("operationID: %s", operationID)

			externalTransactionInfo.OperationID = operationID
			externalTransactionInfo.ExternalTransactionID = externalTransactionID

			syncRequest.ExternalTransactionInfoList = append(syncRequest.ExternalTransactionInfoList, externalTransactionInfo)
		}
		chaincode.GlobalLogger.Infof("ExternalTransactionID", externalTransactionID)
	}

	if len(syncRequest.ExternalTransactionInfoList) == 0 {
		chaincode.GlobalLogger.Info("ExternalTransactionInfoList is empty.")

		return checkExistsResult, nil
	}

	chaincode.GlobalLogger.Infof("Send request 'checkExistsTransactionByList' to transactionCC with list len %s", len(syncRequest.ExternalTransactionInfoList))
	// проверить по списку на существование в канале транзакций
	checkExistsResult, err = this.CheckExistsTransactionByList(&syncRequest)
	if err != nil {
		return nil, err
	}

	chaincode.GlobalLogger.Info("delete all ALREADY_EXISTS cryptocurrencyStore by ExternalTransactionID")
	// пройтись по списку результатов для каждого ExternalTransactionID
	for _, checkExistsExternalTransactionResult := range checkExistsResult.CheckExistsExternalTransactionResultList {
		chaincode.GlobalLogger.Infof("received result for ExternalTransactionID", checkExistsExternalTransactionResult.ExternalTransactionID)
		chaincode.GlobalLogger.Infof("received result status", checkExistsExternalTransactionResult.Status)

		// проверить если стоит статус ALREADY_EXISTS
		if checkExistsExternalTransactionResult.Status == transactionCC.ALREADY_EXISTS {
			// удалить этот ExternalTransactionID из списка транзакций т.к заявка с таким ExternalTransactionID уже обработана
			chaincode.GlobalLogger.Infof("delete from LastImportKeys ExternalTransactionID '%s' because ALREADY_EXISTS", checkExistsExternalTransactionResult.ExternalTransactionID)
			delete(daemonKeys, checkExistsExternalTransactionResult.ExternalTransactionID)
		}
	}

	// устанавливаем измененный map ключей для текущего демона
	lastImportKeys.MapDaemonNameKeys[daemonName.Id] = daemonKeys

	// запись последних импортированных ключей
	err = this.saveLastImportKeys(*lastImportKeys)
	if err != nil {
		return nil, err
	}

	// отправить ответ от чейнкода transactionCC
	return checkExistsResult, nil
}
