package iaccount

import (
	"gitlab.smartblocklab.com/smartcontracts/common/model/requests"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"gitlab.smartblocklab.com/smartcontracts/common/model/accountCC"
)

type IAccountManager interface {
	// invoke
	AddWallet(userID string, newWalletUISigned accountCC.NewWallet, walletId *requests.Id) (chaincode.ValidObject, error)
	AddAccount(userID string, newAccountUISigned accountCC.NewAccount, accountId *requests.Id) (chaincode.ValidObject, error)
	ActivateWallet(userID string, walletId *requests.Id) (*accountCC.Wallet, error)
	BlockWallet(userID string, walletId *requests.Id) (*accountCC.Wallet, error)
	ActivateAccount(userID string, accountId *requests.Id) (*accountCC.Account, error)
	BlockAccount(userID string, accountId *requests.Id) (*accountCC.Account, error)
	// query
	GetAccountByCurrencyType(getAccountByCurrencyTypeArg accountCC.GetAccountByCurrencyTypeArg) (*accountCC.Account, error)
	VerifyUserAccount(userAccountInfo *accountCC.UserAccountInfo) (*accountCC.Account, error)
	CheckAndLoadAccount(userAccountInfo *accountCC.UserAccountInfo) (*accountCC.Account, error)
	GetAllUserWallets(userID *requests.Id) (*accountCC.UserWalletsEnvelope, error)
	GetAccountInfoByAccountID(accountID *requests.Id) (*accountCC.AccountInfo, error)
	GetAccounts(userID *requests.Id) (*accountCC.AccountsEnvelope, error)
	GetWallets(userID *requests.Id) (*accountCC.WalletsEnvelope, error)
}
