package cryptocurrencyCC

import (
	"gitlab.smartblocklab.com/smartcontracts/common/nums"
	"gitlab.smartblocklab.com/smartcontracts/common/model/accountCC"
	"github.com/pkg/errors"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"encoding/json"
	"reflect"
)

// CryptocurrencyStore encapsulates information about
// cryptocurrencies store
type CryptocurrencyStore struct {
	Time                  int64                      `json:"time"`
	ExternalTransactionID string                     `json:"external_transaction_id"`
	OutAccountInfo        *accountCC.UserAccountInfo `json:"out_account_info_id"`
	InAccountInfo         *accountCC.UserAccountInfo `json:"in_account_info_id"`
	Amount                nums.PlainCurrencyNumber   `json:"amount"`
}

func (this CryptocurrencyStore) New() chaincode.ValidObject {
	return &CryptocurrencyStore{}
}

func (this CryptocurrencyStore) CheckBeforeAdd(manager chaincode.IManager) error {
	return nil
}

func (this CryptocurrencyStore) CheckBeforeUpdate(updatedObject chaincode.ValidObject, manager chaincode.IManager) error {
	return nil
}

func (this CryptocurrencyStore) GetKey(manager chaincode.IManager) []string {
	return []string{this.ExternalTransactionID}
}

func (this CryptocurrencyStore) Validate() error {
	if this.Time == 0 {
		return errors.New("Time is empty")
	}

	if len(this.ExternalTransactionID) == 0 {
		return errors.New("ExternalTransactionID is empty")
	}

	if this.InAccountInfo == nil {
		return errors.New("InAccountInfo is empty")
	}

	if len(this.Amount.Amount) == 0 {
		return errors.New("Amount.Amount is empty")
	}

	if len(this.Amount.CurrencyType) == 0 {
		return errors.New("Amount.CurrencyType is empty")
	}

	return nil
}

func Unmarshal(data []byte) (*CryptocurrencyStore, error) {
	var obj CryptocurrencyStore
	err := json.Unmarshal(data, &obj)
	return &obj, err;
}

func Marshal(data *CryptocurrencyStore) ([]byte, error) {
	return json.Marshal(data);
}

func Cast(validObject chaincode.ValidObject) (*CryptocurrencyStore, error) {
	if validObject == nil {
		return nil, nil
	}

	if reflect.ValueOf(validObject).Kind() != reflect.Ptr {
		return nil, errors.New("check validObjectc is not reference")
	}

	casted, ok := validObject.(*CryptocurrencyStore)
	if !ok {
		return nil, errors.New("Problem to get object")
	}

	return casted, nil;
}
