package remote_account

import (
	"encoding/json"
	"gitlab.smartblocklab.com/smartcontracts/common/interfaces/iaccount"
	"gitlab.smartblocklab.com/smartcontracts/common/invoke"
	"gitlab.smartblocklab.com/smartcontracts/common/model/requests"
	"gitlab.smartblocklab.com/smartcontracts/common/model/accountCC"
	"github.com/pkg/errors"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
)

type AccountRemoteManager struct {
	*chaincode.RemoteManagerImpl
}

func GetAccountRemoteManager(iManager chaincode.IManager, channelName string, chaincodeName string) iaccount.IAccountManager {
	remoteManagerImpl := &chaincode.RemoteManagerImpl{IManager: iManager, ChannelName: channelName, ChaincodeName: chaincodeName}
	accountRemoteManager := &AccountRemoteManager{remoteManagerImpl}
	return accountRemoteManager
}

func (this *AccountRemoteManager) AddWallet(userID string, newWalletUISigned accountCC.NewWallet, walletId *requests.Id) (chaincode.ValidObject, error) {
	return nil, errors.New("Failed to make remote call")
}

func (this *AccountRemoteManager) AddAccount(userID string, newAccountUISigned accountCC.NewAccount, accountId *requests.Id) (chaincode.ValidObject, error) {
	return nil, errors.New("Failed to make remote call")
}

func (this *AccountRemoteManager) ActivateWallet(userID string, walletId *requests.Id) (*accountCC.Wallet, error) {
	return nil, errors.New("Failed to make remote call")
}

func (this *AccountRemoteManager) BlockWallet(userID string, walletId *requests.Id) (*accountCC.Wallet, error) {
	return nil, errors.New("Failed to make remote call")
}

func (this *AccountRemoteManager) ActivateAccount(userID string, accountId *requests.Id) (*accountCC.Account, error) {
	return nil, errors.New("Failed to make remote call")
}

func (this *AccountRemoteManager) BlockAccount(userID string, accountId *requests.Id) (*accountCC.Account, error) {
	return nil, errors.New("Failed to make remote call")
}

func (this *AccountRemoteManager) GetAccountByCurrencyType(getAccountByCurrencyTypeArg accountCC.GetAccountByCurrencyTypeArg) (*accountCC.Account, error) {
	if resp := this.SignedInvokeChaincode(this.GetChannelName(), this.GetChaincodeName(), invoke.GetAccountByCurrencyTypeFunction, getAccountByCurrencyTypeArg); resp.Status != 200 {
		return nil, errors.New(resp.Message)
	} else {
		var result accountCC.Account
		if err := json.Unmarshal(resp.Payload, &result); err != nil {
			return nil, errors.New(err.Error())
		} else {
			return &result, nil
		}
	}
}

func (this *AccountRemoteManager) VerifyUserAccount(accountInfo *accountCC.UserAccountInfo) (*accountCC.Account, error) {
	if resp := this.SignedInvokeChaincode(this.GetChannelName(), this.GetChaincodeName(), invoke.VerifyUserAccountFunction, accountInfo); resp.Status != 200 {
		return nil, errors.New(resp.Message)
	} else {
		var result accountCC.Account
		if err := json.Unmarshal(resp.Payload, &result); err != nil {
			return nil, errors.New(err.Error())
		} else {
			return &result, nil
		}
	}
}

func (this *AccountRemoteManager) CheckAndLoadAccount(accountInfo *accountCC.UserAccountInfo) (*accountCC.Account, error) {
	if resp := this.SignedInvokeChaincode(this.GetChannelName(), this.GetChaincodeName(), invoke.CheckAndLoadAccountFunction, accountInfo); resp.Status != 200 {
		return nil, errors.New(resp.Message)
	} else {
		var result accountCC.Account
		if err := json.Unmarshal(resp.Payload, &result); err != nil {
			return nil, errors.New(err.Error())
		} else {
			return &result, nil
		}
	}
}

func (this *AccountRemoteManager) GetAllUserWallets(userID *requests.Id) (*accountCC.UserWalletsEnvelope, error) {
	if resp := this.SignedInvokeChaincode(this.GetChannelName(), this.GetChaincodeName(), invoke.GetAllUserWalletsFunction, userID); resp.Status != 200 {
		return nil, errors.New(resp.Message)
	} else {
		var result accountCC.UserWalletsEnvelope
		if err := json.Unmarshal(resp.Payload, &result); err != nil {
			return nil, errors.New(err.Error())
		} else {
			return &result, nil
		}
	}
}

func (this *AccountRemoteManager) GetAccountInfoByAccountID(accountID *requests.Id) (*accountCC.AccountInfo, error) {
	if resp := this.SignedInvokeChaincode(this.GetChannelName(), this.GetChaincodeName(), invoke.GetAccountInfoByAccountIDFunction, accountID); resp.Status != 200 {
		return nil, errors.New(resp.Message)
	} else {
		var result accountCC.AccountInfo
		if err := json.Unmarshal(resp.Payload, &result); err != nil {
			return nil, errors.New(err.Error())
		} else {
			return &result, nil
		}
	}
}

func (this *AccountRemoteManager) GetAccounts(userID *requests.Id) (*accountCC.AccountsEnvelope, error) {
	if resp := this.SignedInvokeChaincode(this.GetChannelName(), this.GetChaincodeName(), invoke.GetAccountsFunction, userID); resp.Status != 200 {
		return nil, errors.New(resp.Message)
	} else {
		var result accountCC.AccountsEnvelope
		if err := json.Unmarshal(resp.Payload, &result); err != nil {
			return nil, errors.New(err.Error())
		} else {
			return &result, nil
		}
	}
}

func (this *AccountRemoteManager) GetWallets(userID *requests.Id) (*accountCC.WalletsEnvelope, error) {
	if resp := this.SignedInvokeChaincode(this.GetChannelName(), this.GetChaincodeName(), invoke.GetWalletsFunction, userID); resp.Status != 200 {
		return nil, errors.New(resp.Message)
	} else {
		var result accountCC.WalletsEnvelope
		if err := json.Unmarshal(resp.Payload, &result); err != nil {
			return nil, errors.New(err.Error())
		} else {
			return &result, nil
		}
	}
}
