package remote_transaction

import (
	"gitlab.smartblocklab.com/smartcontracts/common/interfaces/itransaction"
	"gitlab.smartblocklab.com/smartcontracts/common/model/transactionCC"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"github.com/pkg/errors"
	"gitlab.smartblocklab.com/smartcontracts/common/model/requests"
	"gitlab.smartblocklab.com/smartcontracts/common/invoke"
	"encoding/json"
)

type TransactionRemoteManagerImpl struct {
	*chaincode.RemoteManagerImpl
}

func GetTransactionRemoteManager(iManager chaincode.IManager, channelName string, chaincodeName string) itransaction.ITransactionManager {
	remoteManagerImpl := &chaincode.RemoteManagerImpl{IManager: iManager, ChannelName: channelName, ChaincodeName: chaincodeName}
	return &TransactionRemoteManagerImpl{remoteManagerImpl}
}

// invoke
func (this TransactionRemoteManagerImpl) LockFundsOnAccount(txnArgs *transactionCC.TxnFundsLockInput) error {
	return errors.New("Failed to make remote call")
}
func (this TransactionRemoteManagerImpl) LockOutFundsOnAccount(txnArgs *transactionCC.OutTxnFundsLockInput) error {
	return errors.New("Failed to make remote call")
}
func (this TransactionRemoteManagerImpl) LockTransferToMarket(txnArgs *transactionCC.OutTxnFundsLockInput) error {
	return errors.New("Failed to make remote call")
}
func (this TransactionRemoteManagerImpl) TransferFunds(id *requests.Id) (*transactionCC.TransferFundsResponseList, error) {
	return nil, errors.New("Failed to make remote call")
}
func (this TransactionRemoteManagerImpl) LoadCryptocurrency(systemUserID string) (*transactionCC.LoadCryptocurrencyResultList, error) {
	return nil, errors.New("Failed to make remote call")
}
func (this TransactionRemoteManagerImpl) LoadMarketFounds(operationID *requests.Id) (*transactionCC.LoadCryptocurrencyResultList, error) {
	return nil, errors.New("Failed to make remote call")
}
func (this TransactionRemoteManagerImpl) WithdrawalMarketFounds(findLockAmountByOperationIDAndUserIdArg transactionCC.FindLockAmountByOperationIDAndUserIdArg) error {
	return errors.New("Failed to make remote call")
}

func (this TransactionRemoteManagerImpl) UnlockFundsOnAccount(id *requests.Id) (*transactionCC.TransferFundsResponseList, error) {
	return nil, errors.New("Failed to make remote call")
}

// query
func (this TransactionRemoteManagerImpl) CheckExistsTransactionByList(syncRequest *transactionCC.SyncRequest) (*transactionCC.CheckExistsResult, error) {
	if resp := this.SignedInvokeChaincode(this.GetChannelName(), this.GetChaincodeName(), invoke.CheckExistsTransactionByListFunction, syncRequest); resp.Status != 200 {
		return nil, errors.New(resp.Message)
	} else {
		var result transactionCC.CheckExistsResult
		if err := json.Unmarshal(resp.Payload, &result); err != nil {
			return nil, errors.New(err.Error())
		} else {
			return &result, nil
		}
	}
}
func (this TransactionRemoteManagerImpl) GetUsersTransaction(userID *requests.Id) (*transactionCC.UserTransactionsList, error) {
	if resp := this.SignedInvokeChaincode(this.GetChannelName(), this.GetChaincodeName(), invoke.GetUsersTransactionFunction, userID); resp.Status != 200 {
		return nil, errors.New(resp.Message)
	} else {
		var result transactionCC.UserTransactionsList
		if err := json.Unmarshal(resp.Payload, &result); err != nil {
			return nil, errors.New(err.Error())
		} else {
			return &result, nil
		}
	}
}
func (this TransactionRemoteManagerImpl) FindLockAmountByOperationIDAndUserId(findLockAmountByOperationIDAndUserIdArg transactionCC.FindLockAmountByOperationIDAndUserIdArg) (*transactionCC.FindLockAmountResult, error) {
	if resp := this.SignedInvokeChaincode(this.GetChannelName(), this.GetChaincodeName(), invoke.FindLockAmountByOperationIDAndUserIdFunction, findLockAmountByOperationIDAndUserIdArg); resp.Status != 200 {
		return nil, errors.New(resp.Message)
	} else {
		var result transactionCC.FindLockAmountResult
		if err := json.Unmarshal(resp.Payload, &result); err != nil {
			return nil, errors.New(err.Error())
		} else {
			return &result, nil
		}
	}
}
func (this TransactionRemoteManagerImpl) AccountCurrencyBalance(accountCurrencyBalanceArg transactionCC.AccountCurrencyBalanceArg) (*transactionCC.AccountBalance, error) {
	if resp := this.SignedInvokeChaincode(this.GetChannelName(), this.GetChaincodeName(), invoke.AccountCurrencyBalanceFunction, accountCurrencyBalanceArg); resp.Status != 200 {
		return nil, errors.New(resp.Message)
	} else {
		var result transactionCC.AccountBalance
		if err := json.Unmarshal(resp.Payload, &result); err != nil {
			return nil, errors.New(err.Error())
		} else {
			return &result, nil
		}
	}
}
